!
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!.COPYRIGHT: (c) 1994 Special Astrophysical Observatory
!.IDENT    : bgd.prg
!.AUTHOR   : A.Yu. Kniazev, V.S.Shergin
!.CALL     : bgd.prg
!.CALL FROM:
!.KEYWORDS : background, continuum, spectroscopy
!.OUTPUT   :
!.PURPOSE  :
!      Extract background from every row of image.
!      Can be used to determine continuum level of the spectrum.
!.USAGE    :
!      Execute as:
! @s bgd  inp_image  out_image  smooth_wnd  noise_level
!         iter_numb  bgd_and_out_type  smooth_mode  smooth_step
!
! where:  inp_image   - input frame name
!         out_image   - output frame name
!         smooth_wnd  - width of suppressing details
!         noise_level - height of suppressing details
!         iter_numb   - number of smooth/suppress steps (ordinary 4 or 5)
!         bgd_and_out_type - two characters (default - "bb") :
!               bgd_type    - a,e,b (for "absorbtion" or "emission" or "both")
!               out_type    -  r - residuals    b - background itself
!         smooth_mode -  <0 - quick-rough mode, sliding window averaging
!                        =0 - slow mode, convolution with gauss-like window
!                        >0 - fitting polynom power ( < 15 :-)
!                        default 1 - linear fitting
!         fitting_step-  step of fitting polynom locations ( < smooth_wnd/2 :-)
!                        default - smooth_wnd/7
!
!.VERSION  : $Header: /u/midas/RAC/Scan_red/proc/RCS/bgd.prg,v 1.2 1995/04/26 07:35:20 midas Exp $
!          : $Revision: 1.2 $
!-------------------------------------------------------------------------
!
	!...input parametres
    define/par p1 ?  I "The_Name_of_Input_Image:"
    define/par p2 ?  I "The_Name_of_Output_Image:"
    define/par p3 ?  N "Smooth_Window_Size:"
    define/par p4 ?  N "Noise_Level_Estimation:"
    define/par p5 5  N "The_Number_of_Iterations:"
    define/par p6 bb C "The_Type_of_Background_and_Output:"
    define/par p7 1  N "Smoothing_Mode:"
    define/par p8 0  N "Step_of_Fitting:"
	!
	!...Lets define our variables
	!
    define/local i/i/1/1 0
    define/local col/i/1/2 0,0
    define/local yes/c/1/10     "h"
    define/local iname/c/1/60    " " all
    define/local oname/c/1/60    " " all
	!
	!...Lets write input parameters in our variables
	!
    i(1) = m$len("{p1}")
    iname(1:{i(1)}) = "{p1}"
    i(1) = m$len("{p2}")
    oname(1:{i(1)}) = "{p2}"
	!
	!...Begin main body
	!
    in_a  = "{iname}"
    in_b  = "{p6}"
    inputi(1) = {p5}
    inputi(2) = {p7}
    inputr(1) = {p3}
    inputr(2) = {p4}
    inputr(3) = {p8}
    out_a = "{oname}"
    out_b = "weight"
    RUN STD_EXE:BGD_SHER
