/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT    (c)  1997   Special Astrophysical Observatory RAS
.IDENT        mbgd.c
.LANGUAGE     C
.AUTHOR       Ts. Georgiev, Rozhen Observatory, BG-4700 Smolyan, Bulgaria
			    tsgeorg@bgearn.bitnet  tsgeorg@phys.acad.bg
	      Shergin V.S., SAO InfLab  vsher@sao.ru
.KEYWORDS    smoothing, median, filtering, backgroung.
.PURPOSE     median decomposition of image to smoothed and residual parts

.COMMENT      Input : IN_A  - input spectra name
		      IN_B  - type of output(r,b)
		      INPUTR(1) - smoothing window size (round aperture diameter)
		      INPUTR(2) - histogram bin size (if 0.0 - calculate it)
		      INPUTR(3) - histogram low excess value
		      INPUTR(4) - histogram hight excess value
				  if INPUTR(3)>=INPUTR(4) - investigate image
				  and try to determine them
		      INPUTI(1) - # of histogram bins (if 0 - calculate it)

	      Output: OUT_A - output spectra name

.VERSION      1.0  94????:   Ts. Georgiev: MDS - median smoothing of INT images,
			     quick (histogram) median with
			     slidin gsquare smoothing window,
			     1st version for MSDOS PCVISTA
	      1.1  9512??:   Ts. Georgiev: MED - decomposition of INT image to
			     median backgroung and residual part,
			     round median filter aperture,
			     improved version for MSDOS PCVISTA
	      2.0  970616:   VSher - version for MIDAS and REAL images
			     (automatically determine histogram parameters)
	      2.1  970708:   VSher - add  histogram bin and excess values
			     setting manually
	      2.1.1 970725:  VSher - fix little bug in mean level calculation
			     and correct LHCUTS setting
---------------------------------------------------------------------*/
#include   <stdio.h>
#include   <math.h>
#include   <midas_def.h>

#define   MNR  2048    /* max image lenght in pixels  */
#define   MNC  2048    /* max image width  in pixels  */
#define   MFW  512     /* max window width for the smoothing */
#define   MHS  32767   /* max histogram size  */

main()
{
    static char infile[64], outfile[64], cbuf[16];
    int stat, nval, naxis, unit, null, imnoa, imnob;
    char ident[TEXT_LEN+1], cunit[16*2+1], text[TEXT_LEN];
    char otype[2];
    int npix[2], inputi[2];
    float inputr[4], port, port2;
    float *idata, *odata, *data;
    float min_data, max_data, mean, dev, hmin, hmax, hbin, hb;
    short *sdat, *mdat;
    double  start[2], step[2];
    float lhcuts[4],i_cuts[4];
    static int *h;
    static int jl[MFW/2+1];           /* limits of the round window */
    int nr,nc,ncf,i,ii,j,jj,k,l,ll,n,fw1,fw,hw,sum,med,hlf,nh;
    double d,s=0.,ss=0.;
    long lo,lzp,lnh;
    fw1=4;

    SCSPRO("mbgd");
	/* Get parameters */
    stat = SCKGETC("IN_A",1,60,&nval,infile);     /* get input frame name  */
    stat = SCKGETC("IN_B",1,1,&nval, otype);    /* 'b'-backgroung 'r'- residual  */
    stat = SCKRDR("INPUTR",1,4,&nval,inputr,&unit,&null);    /*   */
    stat = SCKRDI("INPUTI",1,1,&nval,inputi,&unit,&null);    /*   */
    stat = SCKGETC("OUT_A",1,60,&nval,outfile);   /* get output frame name */
    port = inputr[0];
    hbin = inputr[1];
    hmin = inputr[2];
    hmax = inputr[3];
    nh   = inputi[0];

    SCTPUT("Read and investigate source image. Wait...");

	/* map input data frame and get some information from descriptors */
    stat = SCIGET(infile, D_R4_FORMAT, F_I_MODE, F_IMA_TYPE, 2L,
	   &naxis, npix, start, step, ident, cunit, (char **)&idata, &imnoa);
    if (naxis > 2) SCETER(3L,"Too many dimensions...");
    if (naxis < 2) SCETER(3L,"1-dimensional array ?!");

	/* open output frame and write standard descriptors */
    stat = SCIPUT(outfile, D_R4_FORMAT, F_O_MODE, F_IMA_TYPE,
	   naxis, npix, start, step, ident, cunit, (char **)&odata, &imnob);

    port = fabs(port / step[0]);
    fw1 = (int)(port +0.5);
    nc = npix[0];
    nr = npix[1];
    if (nc <= fw1 || nr <= fw1 ) SCETER(3L,"Too small image...");
    if ( nr>MNR || nc>MNC )      SCETER(3L,"Image too big...");

	/* -- computing the limits of the convolute nucleus -- */
    fw1=abs(fw1);
    if(fw1<2) fw1=2;
    if(fw1>MFW) fw1=MFW;
    fw=fw1;
    hw=fw/2;
    fw=hw*2+1;               /* hw: half-width of the smooth.window */
    ncf=nc+fw;
    port2 = port*port/4;
    n=1;
    for (i=0; i<=hw; i++)  {
	    ii=i*i;       /* j-lim & npix of the nucleus */
	    for (j=0; j<=hw; j++)
	       if(ii+j*j <= port2) {
		    jl[i]=j;
		    if (i>0) n+=4;
	    }
    }
    hlf=n/2+1;
    lnh = (nh==0)? MHS-2 : (nh<1024)? 1024 : nh;
    if (nh==0 && hbin < 1e-20) {
       nh = MHS-2; hbin=0.;
    }
	/* allocate working arrays... */
    if ((h = (int *)malloc((lnh+2)*sizeof(int))) == NULL) {
	sprintf(text,"Can't allocate %d I4-array :-(",(lnh+2));
	SCETER(3L,text);
    }
    if ((sdat = (short *)malloc((nc+fw)*(nr+fw)*sizeof(short))) == NULL) {
	sprintf(text,"Can't allocate %dx%d I2-array :-(",(nc+fw),(nr+fw));
	SCETER(3L,text);
    }
    if(hmin + 1.e-5 >= hmax) {
	   /* investigate image and set histogram parameters: hmin, hmax, hbin */
       min_data = max_data = *(data = idata);
       for (i=0; i<nr; i++)
	   for (j=0; j<nc; j++,data++) {
	       if (*data > max_data ) max_data = *data;
	       if (*data < min_data ) min_data = *data;
	   }
       /* form the first "rough" histogram */
       hmin = lhcuts[2] = min_data;
       hmax = lhcuts[3] = max_data;
       for(jj=0;jj<2;jj++) {
	   hb = (hmax-hmin)/1024;
	   for (l=0; l<1024; l++)
	       h[l]=0;
	   for (data = idata,i=0; i<nr; i++)
	       for (j=0; j<nc; j++,data++) {
		   n = (*data-hmin)/hb;
		   if(n<0 || n>1023)  continue;
		   h[n]++;
	       }
	   /* find  histogram maximum position*/
	   for (l=0,ll=0,n=h[0]; l<1024; l++) {
	       if( h[l] > n ) {
		    ll = l;
		    n = h[l];
	       }
	   }
	   for (i=0,l=0; l<1024; l++)
	       if(h[l] >= h[ll]/2) i++;

	   mean = hmin+ll*hb;
	   dev = i*hb/2.;

/*printf("\n  MIN=%f   MAX=%f   N_BINS=1024   BINSIZE=%f\n",hmin,hmax,hb);
 */
	   /* determine new histogram parameters */
	   n = nc*nr/1000;  /* 0.1% of image pixels */
	   if(jj==0) {
	       for (sum=h[1023],l=1023; sum<n; l--,sum+=h[l]);
	       hmax = lhcuts[1] = hmin + l*hb;
	       for (sum=h[0],l=0; sum<n; l++,sum+=h[l]);
	       hmin = lhcuts[0] = hmin + l*hb;
	   } else {
	       /* find 0.1% histogram level */
		for (k=1,ii=0; ii<n; k++)
		    for (l=0,ii=0; l<1024; l++)
			if( h[l] < k )
			    ii += h[l];

		for (l=ll; h[l]>k && l<1024; l++);
		hmax = lhcuts[1] = hmin + l*hb;
		for (l=ll; h[l]>k && l>0; l--);
		hmin = lhcuts[0] = hmin + l*hb;
	   }
       }
       if (nh>0) {
	  hbin = (hmax-hmin)/nh;
/*printf("  MIN=%f   MAX=%f   N_BINS=%d   BINSIZE=%f\n",hmin,hmax,nh,hbin);
 */
	   /* form the new "precise" histogram */
	   for (l=0; l<=nh; l++)
	       h[l]=0;
	   for (data = idata,i=0; i<nr; i++) {
	       for (j=0; j<nc; j++,data++) {
		   n = (*data-hmin)/hbin;
		   if(n<0)  n=0;
		   if(n>nh) n=nh;
		   h[n]++;
	       }
	   }
	   n = nc*nr/2;
	   med = 0;
	   sum = h[0];
	   while(sum<n) {
	       med++;
	       sum += h[med];
	   }
	   mean = hmin+med*hbin;

	   /* determine number of zero-bins in it */
	   for (n=0,l=0; l<=nh; l++)
	       if(h[l]==0) n++;
	   d = (double)n/(double)nh;
	   if (d>0.5) {            /* if more then 50% zero-bins - correct hishogram...*/
	       d -= 0.03;
	       hmin = mean - (mean-hmin)/(1.-d);
	       hmax = mean + (hmax-mean)/(1.-d);
	       hbin = (hmax-hmin)/nh;
	   }
       } else {
	   nh = (int)((hmax-hmin)/hbin + 0.5);
       }
    } else {
       if (nh==0) {
	  nh = (int)((hmax-hmin)/hbin + 0.5);
       } else  {
	  hbin = (hmax-hmin)/nh;
       }
       /* some statistics on source image... */
       for (l=0; l<=nh; l++)
	   h[l]=0;
       min_data = max_data = *(data = idata);
       for (i=0; i<nr; i++)
	   for (j=0; j<nc; j++,data++) {
	       if (*data > max_data ) max_data = *data;
	       if (*data < min_data ) min_data = *data;
	       n = (*data-hmin)/hbin;
	       if(n<0)  n=0;
	       if(n>nh) n=nh;
	       h[n]++;
	   }
       lhcuts[0] = hmin;
       lhcuts[1] = hmax;
       lhcuts[2] = min_data;
       lhcuts[3] = max_data;
       n = nc*nr/2;
       med = 0;
       sum = h[0];
       while(sum<n) {
	   med++;
	   sum += h[med];
       }
       mean = hmin+med*hbin;
    }
    lnh = nh;
	/* reallocate histogram  array... */
    if ((h = (int *)realloc(h,(lnh+2)*sizeof(int))) == NULL) {
	sprintf(text,"Can't allocate %d I4-array :-(",(lnh+2));
	SCETER(3L,text);
    }

    SCTPUT("Statistics: ");
    sprintf(text,"   DATAMIN=%f DATAMAX=%f MEDIAN=%f ",min_data,max_data,mean);
    SCTPUT(text);

    SCTPUT("Histogram parameters for quick median: ");
    sprintf(text,"   EXCESS=%f,%f   N_BINS=%d   BINSIZE=%f",hmin,hmax,nh,hbin);
    SCTPUT(text);
    sprintf(text,"Median smoothing with round aperture (size %d pixels):",fw);
    SCTPUT(text);

    /* ------ r e w r i t i n g : float -> short -------------------------- */
    for (i=0,k=hw; i<nr; i++,k++) {
	for (j=0,l=hw; j<nc; j++,l++) {
	    lo = (idata[i*nc+j] - hmin) / hbin;
	    if(lo<1) lo=1;
	    if(lo>lnh) lo=lnh;
	    sdat[k*ncf+l] = (short)lo;
	}
	for (j=0; j<hw; j++)  {              /* adding left and right margins */
	    sdat[k*ncf+j] = sdat[k*ncf+nc+j];
	    sdat[k*ncf+nc+hw+j] = sdat[k*ncf+hw+j];
	}
    }
    for (i=0; i<hw; i++)                   /* adding upper and down margins */
	for (j=0; j<nc+2*hw; j++)  {
	    sdat[i*ncf+j] = sdat[(nr+i)*ncf+j];
	    sdat[(nr+hw+i)*ncf+j] = sdat[(hw+i)*ncf+j];
	}

    /* ---------------------------- p r o c e s s --------------------------- */
    for (i=0; i<nr; i++) {              /* i-loop by the image rows  */
printf("line%d (%4.1f%%)\r",i,(float)i*100./nr);fflush(stdout);
	for (j=0; j<nc; j++) {      /* j-loop in the current row */
	    if(j==0)  {
		for (n=0; n<=nh; n++)       /* initialization */
		    h[n]=0;
		for (k=(ii=i-hw)+hw; ii<=i+hw; ii++,k++)   {
		    l=abs(ii-i);
		    ll= jl[l];
		    for (l=(jj=j-ll)+hw; jj<=j+ll; jj++,l++) {
			n = sdat[k*ncf+l];
			h[n]++;
		    }
		}
		med=0;
		sum=h[0];
		while(sum<hlf) {
		    med++;
		    sum += h[med];
		}
	    } /* first med*/

	    else  {
		for (k=(ii=i-hw)+hw; ii<=i+hw; ii++,k++) {  /* loop for the edge colones */
		    l=abs(ii-i);
		    ll= jl[l];                   /* of the current pixel */

		    l = hw+j-ll-1;
		    n = sdat[k*ncf+l];
		    h[n]--;
		    if (n <= med)  sum--;  /* removing */

		    l = hw+j+ll;
		    n = sdat[k*ncf+l];
		    h[n]++;
		    if (n <= med)  sum++;  /* adding */
		}
		while (sum >=hlf) {
		    sum -= h[med];
		    med--;
		}       /* searching */
		while (sum < hlf) {
		    med++;
		    sum += h[med];
		}   /* find curr.med */
	    } /* end of else for curr.j */
	    l = i*nc+j;
	    odata[l] = med*hbin + hmin;           /* WRITING SMOOTH IMAGE */
	    d=idata[l]-odata[l];
	    if (otype[0] == 'r' ) odata[l] = d+mean; /* WRITING RESIDUAL IMAGE */
	    s+=d;
	    ss+=d*d;        /* statistics */
	}  /* end of j-loop */
    } /* end of i-loop */
printf("\n");
    d=(double)nr*(double)nc;
    ss=sqrt((ss-s*s/d)/(d-1.));
    s/=d;
    sprintf(text,"Residuals statistics: Avr=%.3f St_Dev=%.3f",s+mean,ss);
    SCTPUT(text);
close: /* Close all */
    stat = SCDCOP(imnoa,imnob,1L,cbuf);     /*  copy all descriptors */
    CGN_DSCUPD(imnoa,imnob," ");            /* update history */
    SCDRDR (imnoa, "LHCUTS", 1,4, &nval, i_cuts,  &unit, &null);
    if(fabs((double)i_cuts[0]) > 1e-10 || fabs((double)i_cuts[1]) > 1e-10) {
	lhcuts[0] = i_cuts[0];
	lhcuts[1] = i_cuts[1];
    }
    if(fabs((double)i_cuts[2]) > 1e-10 || fabs((double)i_cuts[3]) > 1e-10) {
	lhcuts[2] = i_cuts[2];
	lhcuts[3] = i_cuts[3];
    }
    SCDWRR (imnoa, "LHCUTS", lhcuts, 1,4, &unit);
    min_data = max_data = *(data = odata);
    for (i=0; i<nr; i++)
	for (j=0; j<nc; j++,data++) {
	    if (*data > max_data ) max_data = *data;
	    if (*data < min_data ) min_data = *data;
	}
    lhcuts[2] = min_data;
    lhcuts[3] = max_data;
    SCDWRR (imnob, "LHCUTS", lhcuts, 1,4, &unit);
    SCFCLO(imnoa);
    SCFCLO(imnob);
    SCSEPI();
}
