!
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!.COPYRIGHT: (c) 1997 Special Astrophysical Observatory
!.IDENT    : mbgd.prg
!.AUTHOR   : Ts. Georgiev, V.Shergin
!.CALL     : mbgd.prg
!.CALL FROM:
!.KEYWORDS    smoothing, median, filtering, backgroung.
!.PURPOSE     median decomposition of image to smoothed and residual parts
!.USAGE    :
!      Execute as:
! @s mbgd  inp_image  out_image  smooth_wnd  [out_type] [bins] [lo_exc,hi_exc]
!
! where:  inp_image   - input frame name
!         out_image   - output frame name
!         smooth_wnd  - aperture diameter
!                        in wold coords. (e.g. 10.5)
!                        or in pixels    (e.g. @41 )
!         out_type    - r - residuals    b - background itself (default)
!         bins        - bin size for histogram calculation or no. of bins
!                       to distinguish between the two options, the
!                       character # is used  for no. of bins;
!                       defaulted to #32765;
!         lo_exc,hi_exc - optional low and high excess values, i.e. end of
!                       lower excess bin and start of higher excess bin in
!                       the histogram;
!                       by the default - investigate image and determine
!                       histogram excess values automatically;
!
!.VERSION  : $Header: $
!          : $Revision: 1.0 $
!-------------------------------------------------------------------------
!
	!...input parametres
    define/par p1 ?      I  "The_Name_of_Input_Image:"
    define/par p2 ?      I  "The_Name_of_Output_Image:"
    define/par p3 ?      C  "Smooth_Window_Size:"
    define/par p4 b      C  "The_Type_of__Output:"
    define/par p5 #32765 C  "BinSize_or_#Bins:"
    define/par p6 0.,0.  C  "LowExcess,HightExcess:"
	!
	!...Lets define our variables
	!
    define/local i/i/1/1 0
    define/local iname/c/1/60    " " all
    define/local oname/c/1/60    " " all
    define/local excess/r/1/2    {p6}
	!
	!...Lets write input parameters in our variables
	!
    i(1) = m$len("{p1}")
    iname(1:{i(1)}) = "{p1}"
    i(1) = m$len("{p2}")
    oname(1:{i(1)}) = "{p2}"
	!
	!...Begin main body
	!
    in_a  = "{iname}"
    in_b  = "{p4}"
    if p3(1:1) .eq. "@" then
      inputr(1) = {p3(2:)} * {{iname},step(1)}
    else
      inputr(1) = {p3}
    endif
    if p5(1:1) .eq. "#" then
      inputi(1) = {p5(2:)}
      inputr(2) = 0.
    else
      inputi(1) = 0
      inputr(2) = {p5}
    endif
    inputr(3) = excess(1)
    inputr(4) = excess(2)
    out_a = "{oname}"
    write/keyw history "@s mbgd "      ! for HISTORY updating

!    RUN STD_EXE:MBGD
    RUN MBGD
